import httpx
import typing

from .const import ATMEEX_API_BASE_URL, COMMON_HEADERS

_NOT_AUTHENTICATED_MSG = "Not authenticated. Call client.signin_with_email() or client.request_sms_code() + signin_with_phone() first"


class AtmeexAuth(httpx.Auth):
    requires_response_body = True

    def __init__(self, access_token: str = "", refresh_token: str = ""):
        self._access_token = access_token
        self._refresh_token = refresh_token

    @staticmethod
    def request_phone_code(phone: str) -> httpx.Request:
        """Create request to send SMS code to phone number."""
        payload = {
            "grant_type": "phone_code",
            "phone": phone,
        }
        return httpx.Request(
            "POST", ATMEEX_API_BASE_URL + "/auth/signup", json=payload, headers=COMMON_HEADERS
        )

    @staticmethod
    async def request_phone_code_async(phone: str) -> None:
        """Send SMS code to phone number."""
        async with httpx.AsyncClient() as client:
            request = AtmeexAuth.request_phone_code(phone)
            response = await client.send(request)
            response.raise_for_status()

    @staticmethod
    def signin_with_phone(phone: str, phone_code: str) -> httpx.Request:
        """Create request to sign in with phone and SMS code."""
        payload = {
            "grant_type": "phone_code",
            "phone": phone,
            "phone_code": phone_code,
        }
        return httpx.Request(
            "POST", ATMEEX_API_BASE_URL + "/auth/signin", json=payload, headers=COMMON_HEADERS
        )

    @staticmethod
    async def signin_with_phone_async(phone: str, phone_code: str) -> tuple[str, str]:
        """Sign in with phone and SMS code. Returns (access_token, refresh_token)."""
        async with httpx.AsyncClient() as client:
            request = AtmeexAuth.signin_with_phone(phone, phone_code)
            response = await client.send(request)
            response.raise_for_status()
            data = response.json()
            return data["access_token"], data["refresh_token"]

    @staticmethod
    def signin_with_email(email: str, password: str) -> httpx.Request:
        """Create request to sign in with email and password."""
        payload = {
            "email": email,
            "password": password,
            "grant_type": "basic",
        }
        return httpx.Request(
            "POST", ATMEEX_API_BASE_URL + "/auth/signin", json=payload, headers=COMMON_HEADERS
        )

    @staticmethod
    async def signin_with_email_async(email: str, password: str) -> tuple[str, str]:
        """Sign in with email and password. Returns (access_token, refresh_token)."""
        async with httpx.AsyncClient() as client:
            request = AtmeexAuth.signin_with_email(email, password)
            response = await client.send(request)
            response.raise_for_status()
            data = response.json()
            return data["access_token"], data["refresh_token"]

    def auth_flow(self, request: httpx.Request) -> typing.Generator[httpx.Request, httpx.Response, None]:
        if self._access_token == "":
            raise RuntimeError(_NOT_AUTHENTICATED_MSG)

        request.headers["authorization"] = f"Bearer {self._access_token}"
        response = yield request

        if response.status_code == 401:
            yield from self._refresh_token()
            request.headers["authorization"] = f"Bearer {self._access_token}"
            yield request

    def _refresh_token(self) -> typing.Generator[httpx.Request, httpx.Response, None]:
        if self._refresh_token == "":
            raise RuntimeError(_NOT_AUTHENTICATED_MSG)

        payload = {
            "grant_type": "refresh_token",
            "refresh_token": self._refresh_token,
        }
        response = yield httpx.Request("POST", ATMEEX_API_BASE_URL + "/auth/signin", json=payload, headers=COMMON_HEADERS)
        if response.status_code == 401:
            raise RuntimeError("Session expired. Please sign in again.")
        else:
            self._handle_auth_response(response)

    def _handle_auth_response(self, response: httpx.Response):
        response.raise_for_status()
        data = response.json()
        self._refresh_token = data["refresh_token"]
        self._access_token = data["access_token"]



import httpx

from .auth import AtmeexAuth
from .const import COMMON_HEADERS, ATMEEX_API_BASE_URL
from .device import Device


class AtmeexClient:

    def __init__(self, http_client: httpx.AsyncClient | None = None) -> None:
        self.auth = AtmeexAuth()
        if http_client is not None:
            self.http_client = http_client
            self.http_client.auth = self.auth
            self.http_client.headers.update(COMMON_HEADERS)
            self.http_client.base_url = ATMEEX_API_BASE_URL
        else:
            self.http_client = httpx.AsyncClient(
                auth=self.auth, headers=COMMON_HEADERS, base_url=ATMEEX_API_BASE_URL
            )

    async def request_sms_code(self, phone: str) -> None:
        """Send SMS code to phone number."""
        await AtmeexAuth.request_phone_code_async(phone)

    async def signin_with_phone(self, phone: str, code: str) -> None:
        """Sign in with phone and SMS code."""
        access_token, refresh_token = await AtmeexAuth.signin_with_phone_async(phone, code)
        self.auth._access_token = access_token
        self.auth._refresh_token = refresh_token

    async def signin_with_email(self, email: str, password: str) -> None:
        """Sign in with email and password."""
        access_token, refresh_token = await AtmeexAuth.signin_with_email_async(email, password)
        self.auth._access_token = access_token
        self.auth._refresh_token = refresh_token

    def restore_tokens(self, access_token: str, refresh_token: str) -> None:
        """Restore session from saved tokens."""
        self.auth._access_token = access_token
        self.auth._refresh_token = refresh_token

    @property
    def access_token(self) -> str | None:
        """Get the current access token."""
        return self.auth._access_token

    @property
    def refresh_token(self) -> str | None:
        """Get the current refresh token."""
        return self.auth._refresh_token

    async def get_devices(self):
        resp = await self.http_client.get("/devices")
        devices_list = resp.json()
        try:
            devices = [Device(self.http_client, device_dict) for device_dict in devices_list]
        except Exception as e:
            print(devices_list)
            return []
        return devices

    def set_temp(self, device_id, temp):
        pass



from dataclasses import dataclass, asdict
from typing import Optional

from dacite import from_dict

# u_pwr_on - питание вентилятора
# 
# u_damp_pos:
# 0 - открыта заслонка
# 1 - смешанный режим
# 2 - закрыта заслонка
# например можно послать {"u_pwr_on": false, "u_damp_pos": 0}
# и будет режим пассивного проветривания
# 
# u_temp_room - установка нагрева - температура * 10
# температура от 10 до 30 с шагом 0.5
# установка температуры -1000 выключает обогрев
# 
# u_fan_speed - скорость вентилятора - от 0 до 6

class Model:
    dict = asdict

@dataclass
class DeviceSettingsModel(Model):
    id: int
    device_id: int
    u_pwr_on: bool
    u_fan_speed: int
    u_damp_pos: int
    u_hum_stg: int
    u_temp_room: int
    u_auto: bool
    u_night: bool
    u_cool_mode: bool
    u_night_start: str
    u_night_stop: str
    u_time_zone: Optional[int]

    @staticmethod
    def fromdict(data):
        return from_dict(data_class=DeviceSettingsModel, data=data)

@dataclass
class DeviceSettingsSetModel(Model):
    u_pwr_on: Optional[bool]
    u_fan_speed: Optional[int]
    u_damp_pos: Optional[int]
    u_temp_room: Optional[int]

    def __init__(self, u_pwr_on=None, u_fan_speed=None, u_damp_pos=None, u_temp_room=None):
        self.u_damp_pos = u_damp_pos
        self.u_fan_speed = u_fan_speed
        self.u_pwr_on = u_pwr_on
        self.u_temp_room = u_temp_room

    def dict(self):
        d = asdict(self, dict_factory=lambda x:
                   {k: v for (k, v) in x if v is not None})
        if len(d) == 0:
            raise ValueError("Can't serialize DeviceSettingsSet when every field is None")

        return d

@dataclass
class DeviceConditionModel(Model):
    time: str
    pwr_on: int
    no_water: int
    co2_ppm: int
    temp_in: int
    temp_room: int
    fan_speed: int
    damp_pos: int
    hum_room: int
    hum_stg: int
    firmware_version: str
    server_time: str
    device_id: int
    created_at: str

    @staticmethod
    def fromdict(data):
        return from_dict(data_class=DeviceConditionModel, data=data)

@dataclass
class DeviceModel(Model):
    id: int
    mac: str
    type: int
    name: str
    room_id: int
    owner_id: int
    created_at: str
    socket_id: str
    fw_ver: str
    model: str
    online: Optional[bool]
    settings: DeviceSettingsModel
    condition: Optional[DeviceConditionModel]

    @staticmethod
    def fromdict(data):
        return from_dict(data_class=DeviceModel, data=data)


import traceback
import httpx
from dacite import from_dict

from .models import DeviceModel, DeviceSettingsSetModel

HEATER_DISABLE_TEMP = -1000

class Device:
    model: DeviceModel
    _http_client: httpx.AsyncClient

    def __init__(self, http_client: httpx.Client, data: dict):
        self._http_client = http_client
        self.model = from_dict(data_class=DeviceModel, data=data)

    async def _set_params(self, params: DeviceSettingsSetModel):
        resp = await self._http_client.put(f"/devices/{self.model.id}/params", json=params.dict())
        resp.raise_for_status()
        device_info = resp.json()
        try:
            self.model = from_dict(data_class=DeviceModel, data = device_info)
        except Exception:
            print(traceback.format_exc())
            print(device_info)

    async def set_heat_temp(self, temp: int):
        if (temp < 100 or temp > 300) and temp != HEATER_DISABLE_TEMP:
            raise ValueError(f"set_heat_temp temp not between 100 and 300 and not -1000: {temp}")

        if temp % 10 not in [0, 5]:
            raise ValueError(f"set_heat_temp temp ends not with 0 or 5: {temp}")

        await self._set_params(DeviceSettingsSetModel(u_temp_room=temp))

    async def set_fan_speed(self, fan_speed: int):
        if fan_speed < 0 or fan_speed > 6:
            raise ValueError(f"set_fan_speed fan_speed not between 0 and 6: {fan_speed}")

        await self._set_params(DeviceSettingsSetModel(u_fan_speed=fan_speed))

    async def set_power_and_damp(self, power: bool, damp_pos: int):
        await self._set_params(DeviceSettingsSetModel(u_damp_pos=damp_pos, u_pwr_on=power))

    async def set_power_only(self, power: bool):
        """Set power without changing damper position."""
        await self._set_params(DeviceSettingsSetModel(u_pwr_on=power))

    async def set_damp_pos(self, damp_pos: int):
        """Set damper position: 0=open, 1=mixed, 2=closed"""
        if damp_pos not in [0, 1, 2]:
            raise ValueError(f"set_damp_pos damp_pos must be 0, 1 or 2: {damp_pos}")

        await self._set_params(DeviceSettingsSetModel(u_damp_pos=damp_pos))



from setuptools import setup, find_packages

setup(
    name='atmeexpy',
    version='0.3.0',
    url='https://github.com/anpavlov/atmeexpy',
    author='Andrey Pavlov',
    author_email='dir94@mail.ru',
    description='Atmeex cloud api',
    download_url = '',
    packages=find_packages(),
    install_requires=['dacite >= 1.8.1', 'httpx >= 0.26.0'],
)
